# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Node.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Location_pb2 as Location__pb2
from . import Tag_pb2 as Tag__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Node.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\020ProtoNodeWrapperP\001',
  serialized_pb=b'\n\nNode.proto\x12\x1dorg.openstreetmap.atlas.proto\x1a\x0eLocation.proto\x1a\tTag.proto\"\x8e\x01\n\tProtoNode\x12\n\n\x02id\x18\x01 \x01(\x03\x12>\n\x08location\x18\x02 \x01(\x0b\x32,.org.openstreetmap.atlas.proto.ProtoLocation\x12\x35\n\x04tags\x18\x05 \x03(\x0b\x32\'.org.openstreetmap.atlas.proto.ProtoTagB\x14\x42\x10ProtoNodeWrapperP\x01'
  ,
  dependencies=[Location__pb2.DESCRIPTOR,Tag__pb2.DESCRIPTOR,])




_PROTONODE = _descriptor.Descriptor(
  name='ProtoNode',
  full_name='org.openstreetmap.atlas.proto.ProtoNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='org.openstreetmap.atlas.proto.ProtoNode.id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='location', full_name='org.openstreetmap.atlas.proto.ProtoNode.location', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='org.openstreetmap.atlas.proto.ProtoNode.tags', index=2,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=73,
  serialized_end=215,
)

_PROTONODE.fields_by_name['location'].message_type = Location__pb2._PROTOLOCATION
_PROTONODE.fields_by_name['tags'].message_type = Tag__pb2._PROTOTAG
DESCRIPTOR.message_types_by_name['ProtoNode'] = _PROTONODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoNode = _reflection.GeneratedProtocolMessageType('ProtoNode', (_message.Message,), {
  'DESCRIPTOR' : _PROTONODE,
  '__module__' : 'Node_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoNode)
  })
_sym_db.RegisterMessage(ProtoNode)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



"""
This module defines the pyatlas geometry primitives as well as various helping
functions for manipulating the geometry. These primitives are built using
lat-long locations on the Earth.
"""

import math
import shapely.geometry

# --- Location definition constants ---
_LATITUDE_MIN_DM7 = -900000000
_LATITUDE_MAX_DM7 = 900000000
_LONGITUDE_MIN_DM7 = -1800000000
_LONGITUDE_MAX_DM7 = 1800000000 - 1
# There are 10 million dm7 in a degree
_DM7_PER_DEGREE = 10000000

# --- PolyLine encoding constants ---
# Do not change the precision. It matches the default in the Java implementation.
_PRECISION = 7
_ENCODING_OFFSET_MINUS_ONE = 63
_FIVE_BIT_MASK = 0x1f
_SIXTH_BIT_MASK = 0x20
_BIT_SHIFT = 5
_MAXIMUM_DELTA_LONGITUDE = 180 * pow(10, _PRECISION)


class Boundable(object):
    """
    A Boundable is any geometric object that can be bounded by Rectangle.
    """

    def __init__(self):
        raise NotImplementedError('Boundable should not be instantiated')

    def bounds(self):
        """
        Get the bounding Rectangle of this object.
        """
        raise NotImplementedError('subclass must implement')

    def intersects(self, polygon):
        """
        Check if this Boundable intersects some Polygon.
        """
        raise NotImplementedError('subclass must implement')


class Location(Boundable):
    """
    A latitude-longitude location. This is the building block of the other
    geometric types.

    Uses the dm7 type to represent coordinates.
    dm7 is an integral representation of a decimal degree fixed to 7 places of
    precision. 7 places are enough to specify any location on Earth with
    submeter accuracy.

    Examples:
    45.01 degrees -> 450_100_000 dm7
    -90 degrees -> -900_000_000 dm7
    150.5 degrees -> 1_505_000_000 dm7
    """

    def __init__(self, latitude, longitude):
        """
        Create a new Location with a dm7 latitude and longitude. To create a
        Location with degrees, use the geometry.location_with_degrees() module
        function.
        """
        if not isinstance(latitude, int):
            raise TypeError('latitude must be an integer')
        if not isinstance(longitude, int):
            raise TypeError('longitude must be an integer')

        if latitude > _LATITUDE_MAX_DM7 or latitude < _LATITUDE_MIN_DM7:
            raise ValueError('latitude {} out of range'.format(str(latitude)))
        if longitude > _LONGITUDE_MAX_DM7 or longitude < _LONGITUDE_MIN_DM7:
            raise ValueError('longitude {} out of range'.format(str(longitude)))

        self.latitude = latitude
        self.longitude = longitude

    def __str__(self):
        """
        Get the wkt string representation of this Location. The pair ordering
        of Locations is always (LATITUDE, LONGITUDE)
        """
        shapely_point = location_to_shapely_point_wkt_compat(self)
        return shapely_point.wkt

    def __eq__(self, other):
        """
        Check if two Locations are equal. Equivalent Locations have equal
        latitude and equal longitude.
        """
        return self.latitude == other.latitude and self.longitude == other.longitude

    def __ne__(self, other):
        """
        Check if two Locations are NOT equal.
        """
        return not (self.latitude == other.latitude and self.longitude == other.longitude)

    def __hash__(self):
        """
        Compute a hashcode for this Location.
        """
        hash_value = self.latitude * 31
        hash_value = hash_value * 31 + self.longitude
        return hash_value

    def get_as_packed_int(self):
        """
        Pack this Location into a 64 bit integer. The higher order 32 bits are
        the latitude, the lower order 32 bits are the longitude.
        """
        packed = self.latitude
        packed = packed << 32
        packed = packed | (self.longitude & 0xFFFFFFFF)
        return packed

    def bounds(self):
        """
        Get the bounding Rectangle of this Location.
        """
        return Rectangle(self, self)

    def intersects(self, polygon):
        """
        Check if this Location intersects some Polygon.
        """
        return polygon.fully_geometrically_encloses_location(self)

    def get_latitude(self):
        """
        Get the latitude of this Location as a dm7.
        """
        return self.latitude

    def get_latitude_deg(self):
        """
        Get the latitude of this Location as a degree.
        """
        return dm7_as_degree(self.get_latitude())

    def get_longitude(self):
        """
        Get the longitude of this Location as a dm7.
        """
        return self.longitude

    def get_longitude_deg(self):
        """
        Get the latitude of this Location as a degree.
        """
        return dm7_as_degree(self.get_longitude())


class PolyLine(Boundable):
    """
    A PolyLine is a set of Locations in a specific order.
    """

    def __init__(self, location_list, deep=False):
        """
        Create a new PolyLine given a Location list. By default, it will perform
        a reference copy of the Location list. Can optionally perform a deep copy
        of the list instead.
        """
        if len(location_list) == 0:
            raise ValueError('cannot have an empty PolyLine')
        if deep:
            self.location_list = []
            for point in location_list:
                new_point = Location(point.get_latitude(), point.get_longitude())
                self.location_list.append(new_point)
        else:
            self.location_list = location_list

    def __str__(self):
        """
        Get the wkt string representation of this PolyLine.
        """
        shapely_poly = polyline_to_shapely_linestring_wkt_compat(self)
        return shapely_poly.wkt

    def __eq__(self, other):
        """
        Check if this PolyLine is the same as another PolyLine. Compares their
        internal Location list.
        """
        if len(self.location_list) != len(other.location_list):
            return False
        for point, other_point in zip(self.locations(), other.locations()):
            if not point == other_point:
                return False
        return True

    def __ne__(self, other):
        """
        Check if this PolyLine is NOT the same as another PolyLine.
        """
        if len(self.location_list) != len(other.location_list):
            return True
        for point, other_point in zip(self.locations(), other.locations()):
            if not point == other_point:
                return True
        return False

    def __hash__(self):
        """
        Compute a hashcode for this PolyLine.
        """
        hash_value = 31
        for point in self.locations():
            hash_value = hash_value * 31 + point.__hash__()
        return hash_value

    def compress(self):
        """
        Transform this PolyLine into its compressed representation. The
        compression is based on the MapQuest compressed lat/lon encoding
        found here:
        https://developer.mapquest.com/documentation/common/encode-decode/
        """
        old_latitude = 0
        old_longitude = 0
        encoded = ""
        precision = pow(10, _PRECISION)
        last = Location(0, 0)

        for point in self.locations():
            latitude = int(round(dm7_as_degree(point.latitude) * precision))
            longitude = int(round(dm7_as_degree(point.longitude) * precision))
            encoded += _encode_number(latitude - old_latitude)
            delta_longitude = longitude - old_longitude
            if delta_longitude > _MAXIMUM_DELTA_LONGITUDE:
                raise ValueError(
                    'unable to compress polyline, consecutive points {} and {} too far apart', last,
                    point)
            encoded += _encode_number(delta_longitude)

            old_latitude = latitude
            old_longitude = longitude
            last = point

        if type(encoded) is str:
            encoded = bytes(encoded, 'utf-8')

        return encoded

    def bounds(self):
        """
        Get the bounding Rectangle of this PolyLine.
        """
        return bounds_locations(self.locations())

    def intersects(self, polygon):
        """
        Check if this PolyLine intersects some Polygon.
        """
        return polygon.overlaps_polyline(self)

    def get_locations_list(self):
        """
        Get the underlying Location list for this PolyLine.
        """
        return self.location_list

    def length(self):
        """
        Get the length of this PolyLine along the surface of the Earth, in meters.
        """
        prev_location = None
        sum_distance = 0

        for cur_location in self.locations():
            if prev_location is not None:
                sum_distance += location_haversine_distance(cur_location, prev_location)
            prev_location = cur_location

        return sum_distance

    def locations(self):
        """
        Get a generator for the Locations in this PolyLine.
        """
        for point in self.location_list:
            yield point


class Polygon(PolyLine):
    """
    A special case of PolyLine that has an extra segment between the last and
    first point - effectively a closed PolyLine. The Polygon does not actually
    re-store the last (first) Location. Instead, the API simulates its presence.
    """

    def __init__(self, location_list, deep=False):
        """
        Create a new Polygon given a Location list. By default, it will perform
        a shallow copy of the parameter list. Can optionally perform a deep copy
        of the list.
        """
        super(Polygon, self).__init__(location_list, deep)

    def __str__(self):
        """
        Get the wkt string representation of this Polygon.
        """
        shapely_poly = polygon_to_shapely_polygon_wkt_compat(self)
        return shapely_poly.wkt

    def fully_geometrically_encloses_location(self, location):
        """
        Test if this Polygon fully geometrically encloses a given Location. Will
        return False if the Location lies perfectly on the Polygon's boundary.
        """
        shapely_point = location_to_shapely_point(location)
        shapely_poly_self = polygon_to_shapely_polygon(self)
        return shapely_poly_self.contains(shapely_point)

    def _overlaps_polygon(self, polygon):
        """
        Test if this Polygon overlaps a given Polygon at any point.
        """
        # TODO Shapely differentiates between overlaps and intersects
        # Shapely intersects() allows one to contain the other
        # Shapely overlaps() means they intersect, but neither contains the other
        # which is the right choice here?
        shapely_polyg_self = polygon_to_shapely_polygon(self)
        shapely_polyg_other = polygon_to_shapely_polygon(polygon)
        return shapely_polyg_self.intersects(shapely_polyg_other)

    def overlaps_polyline(self, polyline):
        """
        Test if this Polygon overlaps a given PolyLine at any point.
        """
        # TODO Shapely differentiates between overlaps and intersects
        # Shapely intersects() allows one to contain the other
        # Shapely overlaps() means they intersect, but neither contains the other
        # which is the right choice here?
        shapely_polyline = polyline_to_shapely_linestring(polyline)
        shapely_poly_self = polygon_to_shapely_polygon(self)
        return shapely_poly_self.intersects(shapely_polyline)

    def closed_loop(self):
        """
        Get a generator for the Locations in this Polygon. Will generate the
        first item again at the end, simulating the closedness of the Polygon.
        """
        for point in self.locations():
            yield point
        yield self.location_list[0]

    def intersects(self, polygon):
        """
        Check if this Polygon intersects some other Polygon (ie. overlaps it
        at any point).
        """
        return self._overlaps_polygon(polygon)


class Rectangle(Polygon):
    """
    A special case of Polygon.
    """

    def __init__(self, lower_left, upper_right):
        """
        Create a new Rectangle using a lower left corner Location and an
        upper right corner Location.
        """
        upper_left = Location(upper_right.get_latitude(), lower_left.get_longitude())
        lower_right = Location(lower_left.get_latitude(), upper_right.get_longitude())
        locations = [lower_left, upper_left, upper_right, lower_right]
        super(Rectangle, self).__init__(locations, deep=True)
        self.lower_left = lower_left
        self.upper_right = upper_right

    def get_lower_left(self):
        """
        Get the lower left corner Location of this Rectangle.
        """
        return self.lower_left

    def get_upper_right(self):
        """
        Get the upper right corner Location of this Rectangle.
        """
        return self.upper_right


def location_with_degrees(latitude, longitude):
    """
    Get a new Location with a latitude and longitude specified in degree values.
    """
    latitude = degree_as_dm7(latitude)
    longitude = degree_as_dm7(longitude)
    return Location(latitude, longitude)


def location_from_packed_int(packed_location):
    """
    Decode a Location object from a packed 64 bit integer. See
    Location.get_as_packed_int() for more information.
    """
    longitude = packed_location & 0xFFFFFFFF
    if longitude & 0x80000000 > 0:
        longitude = longitude - (1 << 32)

    latitude = (packed_location >> 32) & 0xFFFFFFFF
    if latitude & 0x80000000 > 0:
        latitude = latitude - (1 << 32)

    return Location(latitude, longitude)


def degree_as_dm7(degree):
    """
    Given a degree, return the equivalent dm7. Does not perform range validation.
    Performs integer conversion of the result.
    """
    return int(round(_DM7_PER_DEGREE * degree))


def dm7_as_degree(dm7):
    """
    Given a dm7, return the equivalent degree. Does not perform range validation.
    """
    return float(dm7) / _DM7_PER_DEGREE


def decompress_polyline(bytestring):
    """
    Given a PolyLine bytestring obtained using PolyLine.compress(),
    decompress it and return it as a PolyLine.
    """
    locations = _decompress_bytestring(bytestring)
    return PolyLine(locations)


def decompress_polygon(bytestring):
    """
    Given a PolyLine bytestring obtained using PolyLine.compress(),
    decompress it and return it as a Polygon.
    """
    locations = _decompress_bytestring(bytestring)
    return Polygon(locations)


def bounds_locations(locations):
    """
    Build a Rectangle that bounds an iterable of Locations.
    """
    yielded_at_least_one = False
    lower_lat = None
    upper_lat = None
    left_lon = None
    right_lon = None

    for location in locations:
        yielded_at_least_one = True
        latitude = location.get_latitude()
        longitude = location.get_longitude()
        if lower_lat is None or latitude < lower_lat:
            lower_lat = latitude
        if upper_lat is None or latitude > upper_lat:
            upper_lat = latitude
        if left_lon is None or longitude < left_lon:
            left_lon = longitude
        if right_lon is None or longitude > right_lon:
            right_lon = longitude

    if not yielded_at_least_one:
        raise ValueError('location iterable must yield at least one value')

    return Rectangle(Location(lower_lat, left_lon), Location(upper_lat, right_lon))


def bounds_atlasentities(entities):
    """
    Build a Rectangle that bounds an iterable of AtlasEntities.
    """
    yielded_at_least_one = False
    lower_lat = None
    upper_lat = None
    left_lon = None
    right_lon = None

    for entity in entities:
        yielded_at_least_one = True
        for location in entity.bounds().locations():
            latitude = location.get_latitude()
            longitude = location.get_longitude()
            if lower_lat is None or latitude < lower_lat:
                lower_lat = latitude
            if upper_lat is None or latitude > upper_lat:
                upper_lat = latitude
            if left_lon is None or longitude < left_lon:
                left_lon = longitude
            if right_lon is None or longitude > right_lon:
                right_lon = longitude

    if not yielded_at_least_one:
        raise ValueError('entity iterable must yield at least one value')

    return Rectangle(Location(lower_lat, left_lon), Location(upper_lat, right_lon))


def location_haversine_distance(location1, location2):
    """
    Given two locations, compute the Haversine distance between them. This is
    the great circle distance between the points on the sphere of the Earth.
    See https://en.wikipedia.org/wiki/Haversine_formula.
    """
    mean_radius = 6371000  # meters
    lat1 = convert_to_radians(location1.get_latitude_deg())
    lat2 = convert_to_radians(location2.get_latitude_deg())
    delta_lat = lat2 - lat1
    delta_lon = convert_to_radians(location2.get_longitude_deg()) - convert_to_radians(
        location1.get_longitude_deg())

    hav = (math.sin(delta_lat / 2)**
           2) + math.cos(lat1) * math.cos(lat2) * (math.sin(delta_lon / 2)**2)
    constant = 2 * math.atan2(math.sqrt(hav), math.sqrt(1 - hav))

    return constant * mean_radius


def convert_to_radians(degree):
    """
    Convert an angle in degrees to the equivalent angle in radians.
    """
    return degree * (math.pi / 180)


def convert_to_degrees(radian):
    """
    Convert an angle in radians to the equivalent angle in degrees.
    """
    return radian * (180 / math.pi)


def boundable_to_shapely_box(boundable):
    """
    Convert a pyatlas Boundable type to its Shapely Polygon representation.
    The Shapely Polygon will always be a rectangle.
    """
    return polygon_to_shapely_polygon(boundable.bounds())


def polygon_to_shapely_polygon(polygon):
    """
    Convert a Polygon to its Shapely Polygon representation.
    """
    shapely_points = []
    for location in polygon.locations():
        shapely_points.append(location_to_shapely_point(location))

    return shapely.geometry.Polygon(shapely.geometry.LineString(shapely_points))


def polygon_to_shapely_polygon_wkt_compat(polygon):
    """
    Convert a Polygon to its Shapely Polygon representation but with WKT
    compatible coordinates.
    """
    shapely_points = []
    for location in polygon.locations():
        shapely_points.append(location_to_shapely_point_wkt_compat(location))

    return shapely.geometry.Polygon(shapely.geometry.LineString(shapely_points))


def location_to_shapely_point(location):
    """
    Convert a Location to its Shapely Point representation.
    """
    latitude = location.get_latitude()
    longitude = location.get_longitude()

    return shapely.geometry.Point(latitude, longitude)


def location_to_shapely_point_wkt_compat(location):
    """
    Convert a Location to its Shapely Point representation but with WKT
    compatible coordinates.
    """
    latitude = location.get_latitude_deg()
    longitude = location.get_longitude_deg()

    return shapely.geometry.Point(longitude, latitude)


def polyline_to_shapely_linestring(polyline):
    """
    Convert a PolyLine to its Shapely LineString representation.
    """
    shapely_points = []
    for location in polyline.locations():
        shapely_points.append(location_to_shapely_point(location))

    return shapely.geometry.LineString(shapely_points)


def polyline_to_shapely_linestring_wkt_compat(polyline):
    """
    Convert a PolyLine to its Shapely LineString representation but with
    WKT compatible coordinates.
    """
    shapely_points = []
    for location in polyline.locations():
        shapely_points.append(location_to_shapely_point_wkt_compat(location))

    return shapely.geometry.LineString(shapely_points)


def _encode_number(number):
    """
    Encode a number as a unicode character.
    """
    number = number << 1
    if number < 0:
        number = ~number
    encoded = ""
    while number >= _SIXTH_BIT_MASK:
        code_point = (_SIXTH_BIT_MASK | number & _FIVE_BIT_MASK) + _ENCODING_OFFSET_MINUS_ONE
        encoded += chr(code_point)
        number = _urshift32(number, _BIT_SHIFT)
    encoded += chr(number + _ENCODING_OFFSET_MINUS_ONE)
    return encoded


def _decompress_bytestring(bytestring):
    """
    Reverse the compression algorithm in PolyLine.compress().
    """
    precision = pow(10, -1 * _PRECISION)
    length = len(bytestring)
    index = 0
    latitude = 0
    longitude = 0
    locations = []

    while index < length:
        shift = 0
        result = 0
        while True:
            byte_encoded = bytestring[index] - _ENCODING_OFFSET_MINUS_ONE
            result |= (byte_encoded & _FIVE_BIT_MASK) << shift
            shift += _BIT_SHIFT
            index += 1
            if byte_encoded < _SIXTH_BIT_MASK:
                break

        if result & 1 > 0:
            delta_latitude = ~(_urshift32(result, 1))
        else:
            delta_latitude = _urshift32(result, 1)
        latitude += delta_latitude

        shift = 0
        result = 0
        while True:
            byte_encoded = bytestring[index] - _ENCODING_OFFSET_MINUS_ONE
            result |= (byte_encoded & _FIVE_BIT_MASK) << shift
            shift += _BIT_SHIFT
            index += 1
            if byte_encoded < _SIXTH_BIT_MASK:
                break

        if result & 1 > 0:
            delta_longitude = ~(_urshift32(result, 1))
        else:
            delta_longitude = _urshift32(result, 1)
        longitude += delta_longitude

        latitude = latitude * precision
        longitude = longitude * precision

        # convert lat/lon to dm7
        latitude = degree_as_dm7(latitude)
        longitude = degree_as_dm7(longitude)

        locations.append(Location(latitude, longitude))

    return locations


def _urshift32(to_shift, shift_amount):
    """
    Perform a 32 bit unsigned right shift (drag in leading 0s).
    """
    return (to_shift % 0x100000000) >> shift_amount



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Line.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Location_pb2 as Location__pb2
from . import Tag_pb2 as Tag__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Line.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\020ProtoLineWrapperP\001',
  serialized_pb=b'\n\nLine.proto\x12\x1dorg.openstreetmap.atlas.proto\x1a\x0eLocation.proto\x1a\tTag.proto\"\x91\x01\n\tProtoLine\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x41\n\x0bshapePoints\x18\x02 \x03(\x0b\x32,.org.openstreetmap.atlas.proto.ProtoLocation\x12\x35\n\x04tags\x18\x03 \x03(\x0b\x32\'.org.openstreetmap.atlas.proto.ProtoTagB\x14\x42\x10ProtoLineWrapperP\x01'
  ,
  dependencies=[Location__pb2.DESCRIPTOR,Tag__pb2.DESCRIPTOR,])




_PROTOLINE = _descriptor.Descriptor(
  name='ProtoLine',
  full_name='org.openstreetmap.atlas.proto.ProtoLine',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='org.openstreetmap.atlas.proto.ProtoLine.id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shapePoints', full_name='org.openstreetmap.atlas.proto.ProtoLine.shapePoints', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='org.openstreetmap.atlas.proto.ProtoLine.tags', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=73,
  serialized_end=218,
)

_PROTOLINE.fields_by_name['shapePoints'].message_type = Location__pb2._PROTOLOCATION
_PROTOLINE.fields_by_name['tags'].message_type = Tag__pb2._PROTOTAG
DESCRIPTOR.message_types_by_name['ProtoLine'] = _PROTOLINE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoLine = _reflection.GeneratedProtocolMessageType('ProtoLine', (_message.Message,), {
  'DESCRIPTOR' : _PROTOLINE,
  '__module__' : 'Line_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoLine)
  })
_sym_db.RegisterMessage(ProtoLine)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProtoLongArray.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ProtoLongArray.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\025ProtoLongArrayWrapperP\001',
  serialized_pb=b'\n\x14ProtoLongArray.proto\x12\x1dorg.openstreetmap.atlas.proto\"0\n\x0eProtoLongArray\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x65lements\x18\x02 \x03(\x03\x42\x19\x42\x15ProtoLongArrayWrapperP\x01'
)




_PROTOLONGARRAY = _descriptor.Descriptor(
  name='ProtoLongArray',
  full_name='org.openstreetmap.atlas.proto.ProtoLongArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='org.openstreetmap.atlas.proto.ProtoLongArray.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='elements', full_name='org.openstreetmap.atlas.proto.ProtoLongArray.elements', index=1,
      number=2, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=55,
  serialized_end=103,
)

DESCRIPTOR.message_types_by_name['ProtoLongArray'] = _PROTOLONGARRAY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoLongArray = _reflection.GeneratedProtocolMessageType('ProtoLongArray', (_message.Message,), {
  'DESCRIPTOR' : _PROTOLONGARRAY,
  '__module__' : 'ProtoLongArray_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoLongArray)
  })
_sym_db.RegisterMessage(ProtoLongArray)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



"""
This module defines the spatial index class for use by the Atlas.
"""

import ctypes
import shapely.geometry
from shapely.geos import lgeos as _lgeos

import pyatlas.geometry


class SpatialIndex(object):
    """
    An optimized container class for making spatial queries on AtlasEntities.
    The Atlas will automatically construct the SpatialIndices it needs as it
    receives queries, so it is unlikely you will ever need to create instances
    of this class manually.
    """

    def __init__(self, parent_atlas, entity_type, initial_entities=None):
        """
        Create a new SpatialIndex. Requires a reference to the parent Atlas of
        this index, as well as the EntityType of the AtlasEntities it will store.
        Can optionally accept an iterable of AtlasEntities with which to initialize
        the index.

        In order to start using the index, one must specify which backing tree
        implementation to use.
        """
        self.tree = None
        self.atlas = parent_atlas
        self.entity_type = entity_type
        self.initial_entities = initial_entities

    def initialize_rtree(self):
        """
        Initialize an R-tree to back this SpatialIndex. The underlying R-tree
        implementation is immutable, so repeated calls to add() will degrade
        performance. For more information, see the documentation in the _RTree class.
        """
        self.tree = _RTree(self.initial_entities)

    def initialize_quadtree(self):
        """
        Intitialize a quadtree to back this SpatialIndex.
        CURRENTLY UNIMPLEMENTED
        """
        # TODO implement the quadtree
        raise NotImplementedError('quadtree currently not implemented')

    def add(self, entity):
        """
        Insert an AtlasEntity into the index.
        """
        if self.tree is None:
            raise ValueError('must select a tree implementation before using')

        if entity.bounds() is not None:
            self.tree.add(entity)
        else:
            raise ValueError('bounds cannot be None')

    def get(self, bounds, predicate=lambda e: True):
        """
        Get a frozenset of AtlasEntities that are within or intersecting some bounds.
        Can optionally accept a matching predicate function.
        """
        if self.tree is None:
            raise ValueError('must select a tree implementation before using')

        results = []
        for item_index in self.tree.get(bounds):
            result = self.atlas.entity(item_index, self.entity_type)
            if predicate(result):
                results.append(result)

        return frozenset(results)


class _RTree(object):
    """
    A wrapper class for the _CustomSTRtree implementation, which calls into the
    native GEOS library using machinery from Shapely.
    
    This class stores raw AtlasEntity identifiers without any
    EntityType information. For this reason, users of _RTree should avoid adding
    AtlasEntities with different EntityTypes to the same tree.
    
    Note also that the underlying tree implementation (GEOS R-tree) this class uses
    is immutable. _RTree simulates mutability by maintaining a parallel list of
    elements, and rebuilding the underlying tree on each add. For this reason,
    extensive use of the add() method is not recommended. For more information on
    the immutability of the GEOS R-tree, check out this class in the GEOS codebase:
    https://github.com/OSGeo/geos/blob/master/src/index/strtree/AbstractSTRtree.cpp
    """

    def __init__(self, initial_entities=None):
        """
        Create a new _RTree, optionally with an iterable of initial AtlasEntities.
        """
        self.contents = []
        self.tree = None

        if initial_entities is not None:
            for entity in initial_entities:
                self.contents.append(entity)
            self._construct_tree_from_contents()

    def _construct_tree_from_contents(self):
        """
        Use the tree's contents list (of AtlasEntities) to rebuild the backing
        _CustomSTRtree.
        """
        contents_shapely_format = [
            pyatlas.geometry.boundable_to_shapely_box(entity) for entity in self.contents
        ]

        # pack the arguments in format expected by the _CustomSTRtree
        hacktree_arguments = []
        for entity, cont in zip(self.contents, contents_shapely_format):
            hacktree_arguments.append((entity.get_identifier(), cont))

        self.tree = _CustomSTRtree(hacktree_arguments)

    def add(self, entity):
        """
        Insert an AtlasEntity into the _RTree. The underlying _CustomSTRtree
        (which trivially wraps the GEOS STRtree) is immutable once "built", so
        this method forces a rebuild of the entire tree. The STRtree is "built"
        once any type of query has been made on it.
        """
        self.contents.append(entity)
        self._construct_tree_from_contents()

    def get(self, boundable):
        """
        Given a Boundable object, get a list of the identifiers of all
        AtlasEntities within the bounds of the Boundable.
        """
        if self.tree is not None:
            boundable = pyatlas.geometry.boundable_to_shapely_box(boundable)
            return self.tree.get(boundable)
        else:
            raise ValueError('R-tree is empty')


class _CustomSTRtree(object):
    """
    Hack re-implementation of the shapely STRtree. Changes the behaviour of the
    STRtree to allow for insertion of the entity atlas identifier (type long).
    """

    def __init__(self, items):
        """
        Parameter items is a list of tuples, where each tuple lookes like:
        (entity_id: long, entity_geometry: shapely.geometry.polygon)
        """
        self._n_geoms = len(items)

        self._tree_handle = shapely.geos.lgeos.GEOSSTRtree_create(max(4, len(items)))
        for item in items:
            _lgeos.GEOSSTRtree_insert(self._tree_handle, item[1]._geom,
                                      ctypes.py_object(int(item[0])))

        geoms = [item[1] for item in items]
        self._geoms = list(geoms)

    def __del__(self):
        if self._tree_handle is not None:
            _lgeos.GEOSSTRtree_destroy(self._tree_handle)
            self._tree_handle = None

    def get(self, geom):
        """
        Get a list of identifiers of AtlasEntities whose bounds intersect the
        bounds defined by the geom parameter.
        """
        if self._n_geoms == 0:
            return []

        result = []

        def callback(item, userdata):
            identifier = ctypes.cast(item, ctypes.py_object).value
            result.append(identifier)

        _lgeos.GEOSSTRtree_query(self._tree_handle, geom._geom, _lgeos.GEOSQueryCallback(callback),
                                 None)

        return result



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProtoIntegerStringDictionary.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ProtoIntegerStringDictionary.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B#ProtoIntegerStringDictionaryWrapperP\001',
  serialized_pb=b'\n\"ProtoIntegerStringDictionary.proto\x12\x1dorg.openstreetmap.atlas.proto\"T\n\x1cProtoIntegerStringDictionary\x12\x14\n\x0c\x63urrentIndex\x18\x01 \x01(\x05\x12\x0f\n\x07indexes\x18\x02 \x03(\x05\x12\r\n\x05words\x18\x03 \x03(\tB\'B#ProtoIntegerStringDictionaryWrapperP\x01'
)




_PROTOINTEGERSTRINGDICTIONARY = _descriptor.Descriptor(
  name='ProtoIntegerStringDictionary',
  full_name='org.openstreetmap.atlas.proto.ProtoIntegerStringDictionary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='currentIndex', full_name='org.openstreetmap.atlas.proto.ProtoIntegerStringDictionary.currentIndex', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indexes', full_name='org.openstreetmap.atlas.proto.ProtoIntegerStringDictionary.indexes', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='words', full_name='org.openstreetmap.atlas.proto.ProtoIntegerStringDictionary.words', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=69,
  serialized_end=153,
)

DESCRIPTOR.message_types_by_name['ProtoIntegerStringDictionary'] = _PROTOINTEGERSTRINGDICTIONARY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoIntegerStringDictionary = _reflection.GeneratedProtocolMessageType('ProtoIntegerStringDictionary', (_message.Message,), {
  'DESCRIPTOR' : _PROTOINTEGERSTRINGDICTIONARY,
  '__module__' : 'ProtoIntegerStringDictionary_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoIntegerStringDictionary)
  })
_sym_db.RegisterMessage(ProtoIntegerStringDictionary)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



"""
This module defines the AtlasMetaData container type.
"""


class AtlasMetaData(object):
    """
    Container class for an Atlas's metadata.

    Readable metadata fields:
    number_of_points (long)
    number_of_lines (long)
    number_of_areas (long)
    number_of_nodes (long)
    number_of_edges (long)
    number_of_relations (long)
    original (bool)
    code_version (string)
    data_version (string)
    country (string)
    shard_name (string)
    tags (dict)
    """

    def __init__(self):
        self.number_of_edges = 0
        self.number_of_nodes = 0
        self.number_of_areas = 0
        self.number_of_lines = 0
        self.number_of_points = 0
        self.number_of_relations = 0
        self.original = False
        self.code_version = ""
        self.data_version = ""
        self.country = ""
        self.shard_name = ""
        self.tags = {}


def _get_atlas_metadata_from_proto(proto_atlas_metadata):
    """
    Take a decoded ProtoAtlasMetaData object and turn
    it into a more user friendly AtlasMetaData object.
    """
    new_atlas_metadata = AtlasMetaData()
    new_atlas_metadata.number_of_edges = proto_atlas_metadata.edgeNumber
    new_atlas_metadata.number_of_nodes = proto_atlas_metadata.nodeNumber
    new_atlas_metadata.number_of_areas = proto_atlas_metadata.areaNumber
    new_atlas_metadata.number_of_lines = proto_atlas_metadata.lineNumber
    new_atlas_metadata.number_of_points = proto_atlas_metadata.pointNumber
    new_atlas_metadata.number_of_relations = proto_atlas_metadata.relationNumber
    new_atlas_metadata.original = proto_atlas_metadata.original
    new_atlas_metadata.code_version = proto_atlas_metadata.codeVersion
    new_atlas_metadata.data_version = proto_atlas_metadata.dataVersion
    new_atlas_metadata.country = proto_atlas_metadata.country
    new_atlas_metadata.shard_name = proto_atlas_metadata.shardName

    # convert prototags and fill the tag dict
    for proto_tag in proto_atlas_metadata.tags:
        new_atlas_metadata.tags[proto_tag.key] = proto_tag.value

    return new_atlas_metadata



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Relation.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Tag_pb2 as Tag__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Relation.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\024ProtoRelationWrapperP\001',
  serialized_pb=b'\n\x0eRelation.proto\x12\x1dorg.openstreetmap.atlas.proto\x1a\tTag.proto\"\xf5\x02\n\rProtoRelation\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x35\n\x04tags\x18\x02 \x03(\x0b\x32\'.org.openstreetmap.atlas.proto.ProtoTag\x12H\n\x05\x62\x65\x61ns\x18\x03 \x03(\x0b\x32\x39.org.openstreetmap.atlas.proto.ProtoRelation.RelationBean\x1a\x84\x01\n\x0cRelationBean\x12\x10\n\x08memberId\x18\x01 \x01(\x03\x12\x12\n\nmemberRole\x18\x02 \x01(\t\x12N\n\nmemberType\x18\x03 \x01(\x0e\x32:.org.openstreetmap.atlas.proto.ProtoRelation.ProtoItemType\"P\n\rProtoItemType\x12\x08\n\x04NODE\x10\x00\x12\x08\n\x04\x45\x44GE\x10\x01\x12\x08\n\x04\x41REA\x10\x02\x12\x08\n\x04LINE\x10\x03\x12\t\n\x05POINT\x10\x04\x12\x0c\n\x08RELATION\x10\x05\x42\x18\x42\x14ProtoRelationWrapperP\x01'
  ,
  dependencies=[Tag__pb2.DESCRIPTOR,])



_PROTORELATION_PROTOITEMTYPE = _descriptor.EnumDescriptor(
  name='ProtoItemType',
  full_name='org.openstreetmap.atlas.proto.ProtoRelation.ProtoItemType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NODE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EDGE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AREA', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LINE', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='POINT', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RELATION', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=354,
  serialized_end=434,
)
_sym_db.RegisterEnumDescriptor(_PROTORELATION_PROTOITEMTYPE)


_PROTORELATION_RELATIONBEAN = _descriptor.Descriptor(
  name='RelationBean',
  full_name='org.openstreetmap.atlas.proto.ProtoRelation.RelationBean',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='memberId', full_name='org.openstreetmap.atlas.proto.ProtoRelation.RelationBean.memberId', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memberRole', full_name='org.openstreetmap.atlas.proto.ProtoRelation.RelationBean.memberRole', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memberType', full_name='org.openstreetmap.atlas.proto.ProtoRelation.RelationBean.memberType', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=220,
  serialized_end=352,
)

_PROTORELATION = _descriptor.Descriptor(
  name='ProtoRelation',
  full_name='org.openstreetmap.atlas.proto.ProtoRelation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='org.openstreetmap.atlas.proto.ProtoRelation.id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='org.openstreetmap.atlas.proto.ProtoRelation.tags', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='beans', full_name='org.openstreetmap.atlas.proto.ProtoRelation.beans', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PROTORELATION_RELATIONBEAN, ],
  enum_types=[
    _PROTORELATION_PROTOITEMTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=61,
  serialized_end=434,
)

_PROTORELATION_RELATIONBEAN.fields_by_name['memberType'].enum_type = _PROTORELATION_PROTOITEMTYPE
_PROTORELATION_RELATIONBEAN.containing_type = _PROTORELATION
_PROTORELATION.fields_by_name['tags'].message_type = Tag__pb2._PROTOTAG
_PROTORELATION.fields_by_name['beans'].message_type = _PROTORELATION_RELATIONBEAN
_PROTORELATION_PROTOITEMTYPE.containing_type = _PROTORELATION
DESCRIPTOR.message_types_by_name['ProtoRelation'] = _PROTORELATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoRelation = _reflection.GeneratedProtocolMessageType('ProtoRelation', (_message.Message,), {

  'RelationBean' : _reflection.GeneratedProtocolMessageType('RelationBean', (_message.Message,), {
    'DESCRIPTOR' : _PROTORELATION_RELATIONBEAN,
    '__module__' : 'Relation_pb2'
    # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoRelation.RelationBean)
    })
  ,
  'DESCRIPTOR' : _PROTORELATION,
  '__module__' : 'Relation_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoRelation)
  })
_sym_db.RegisterMessage(ProtoRelation)
_sym_db.RegisterMessage(ProtoRelation.RelationBean)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



"""
This module defines helpful functions to extract information from Atlas identifiers.
"""

# Country code and way sectioned identifiers are 3 decimal digits
_IDENTIFIER_SCALE = 1000


def get_osm_identifier(full_atlas_identifier):
    """
     Get the OSM identifier from the full Atlas identifier by removing the
     country code and way section index.

     Example:
     Atlas ID: 222222001003 would return OSM ID: 222222
    """
    full_atlas_identifier = abs(full_atlas_identifier)
    return full_atlas_identifier // (_IDENTIFIER_SCALE * _IDENTIFIER_SCALE)


def get_country_code(full_atlas_identifier):
    """
    Get the country code from the full Atlas identifier.

    Example:
    Atlas ID: 222222001003 would return country code: 1
    """
    full_atlas_identifier = abs(full_atlas_identifier)
    return (full_atlas_identifier // _IDENTIFIER_SCALE) % _IDENTIFIER_SCALE


def get_way_section_index(full_atlas_identifier):
    """
    Get the way section index from the full Atlas identifier.

    Example:
    Atlas ID: 222222001003 would return index: 3
    """
    full_atlas_identifier = abs(full_atlas_identifier)
    return full_atlas_identifier % _IDENTIFIER_SCALE



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Tag.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='Tag.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\017ProtoTagWrapperP\001',
  serialized_pb=b'\n\tTag.proto\x12\x1dorg.openstreetmap.atlas.proto\"&\n\x08ProtoTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\tB\x13\x42\x0fProtoTagWrapperP\x01'
)




_PROTOTAG = _descriptor.Descriptor(
  name='ProtoTag',
  full_name='org.openstreetmap.atlas.proto.ProtoTag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='org.openstreetmap.atlas.proto.ProtoTag.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='org.openstreetmap.atlas.proto.ProtoTag.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=44,
  serialized_end=82,
)

DESCRIPTOR.message_types_by_name['ProtoTag'] = _PROTOTAG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoTag = _reflection.GeneratedProtocolMessageType('ProtoTag', (_message.Message,), {
  'DESCRIPTOR' : _PROTOTAG,
  '__module__' : 'Tag_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoTag)
  })
_sym_db.RegisterMessage(ProtoTag)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



"""
This module defines the Atlas entity types. Entities are features that can be
queried from the Atlas, so things like Nodes, Lines, Relations, etc. The Atlas
entities themselves are flyweight classes, and simply store references into the
Atlas feature arrays.

In general, the entity classes in this module should not be instantiated
directly. Instead, entity objects should be obtained through the appropriate
Atlas API method.
"""

import pyatlas.identifier_converters
import pyatlas.geometry


class EntityType(object):
    """
    An enum for AtlasEntity types. Valid settings are NODE, EDGE, AREA, LINE,
    POINT, and RELATION.
    """

    def __init__(self):
        raise NotImplementedError

    # these MUST match the Java implementation for serialization compatibility
    NODE = 0
    EDGE = 1
    AREA = 2
    LINE = 3
    POINT = 4
    RELATION = 5

    _strs = {
        NODE: "NODE",
        EDGE: "EDGE",
        AREA: "AREA",
        LINE: "LINE",
        POINT: "POINT",
        RELATION: "RELATION",
    }


class AtlasEntity(pyatlas.geometry.Boundable):
    """
    A tagged, located entity in an Atlas. Can be a member of a relation.
    An AtlasEntity should not be instantiated directly. Use one of the
    appropriate sub-classes.
    """

    def __init__(self, parent_atlas):
        """
        AtlasEntity should not be instantiated directly.
        """
        self.parent_atlas = parent_atlas

    def __eq__(self, other):
        """
        Determine if this AtlasEntity is equal to another. Two entities are
        considered equal if they have the same identifier and the same type.
        """
        return self.get_identifier() == other.get_identifier() and self.get_type(
        ) == other.get_type()

    def __ne__(self, other):
        """
        Determine if this AtlasEntity is NOT equal to another. Inverse of the
        comparison made by the __eq__() method.
        """
        return not (self.get_identifier() == other.get_identifier()
                    and self.get_type() == other.get_type())

    def __hash__(self):
        """
        Compute a hashcode for this AtlasEntity.
        """
        return self.get_identifier() * 31 + self.get_type()

    def get_identifier(self):
        """
        Get the Atlas identifier of this AtlasEntity.
        """
        raise NotImplementedError('subclass must implement')

    def get_tags(self):
        """
        Get a dictionary of this AtlasEntity's tags.
        """
        raise NotImplementedError('subclass must implement')

    def bounds(self):
        """
        Compute the bounding Rectangle of this AtlasEntity.
        """
        raise NotImplementedError('subclass must implement')

    def intersects(self, polygon):
        """
        Check if this AtlasEntity intersects some Polygon.
        """
        raise NotImplementedError('subclass must implement')

    def relations(self):
        """
        Get the set of Relations of which this AtlasEntity is a member.
        """
        raise NotImplementedError('subclass must implement')

    def get_type(self):
        """
        Get the EntityType of this AtlasEntity
        """
        raise NotImplementedError('subclass must implement')

    def get_osm_identifier(self):
        """
        Get the OSM identifier of this AtlasEntity.
        """
        atlas_id = self.get_identifier()
        return pyatlas.identifier_converters.get_osm_identifier(atlas_id)

    def get_parent_atlas(self):
        """
        Get the Atlas that contains this AtlasEntity.
        """
        return self.parent_atlas

    def _get_relations_helper(self, relation_map, index):
        """
        Subclasses of AtlasEntity can use this helper function to
        avoid code duplication in their relations() implementations.
        """
        relation_set = set()

        if index not in relation_map:
            return relation_set

        for relation_index in relation_map[index]:
            relation = Relation(self.get_parent_atlas(), relation_index)
            relation_set.add(relation)

        return frozenset(relation_set)


class Point(AtlasEntity):
    """
    An Atlas Point. Points are non-navigable.
    """

    def __init__(self, parent_atlas, index):
        """
        Constuct a new Point. This should not be called directly.
        """
        super(Point, self).__init__(parent_atlas)
        self.index = index

    def __str__(self):
        """
        Get a string representation of this Point.
        """
        result = '[ '
        result += 'Point: id=' + str(self.get_identifier())
        result += ', geom=' + str(self.as_location())
        result += ', tags=' + str(self.get_tags())

        string = ''
        for relation in self.relations():
            string += str(relation.get_identifier()) + ','
        result += ', relations=[' + string + ']'

        result += ' ]'
        return result

    def get_identifier(self):
        """
        Get the Atlas identifier of this Point.
        """
        return self.get_parent_atlas()._get_pointIdentifiers().elements[self.index]

    def as_location(self):
        """
        Get the Location of this Point.
        """
        long_location = self.get_parent_atlas()._get_pointLocations().elements[self.index]
        return pyatlas.geometry.location_from_packed_int(long_location)

    def get_tags(self):
        """
        Get a dictionary of this Point's tags.
        """
        point_tag_store = self.get_parent_atlas()._get_pointTags()
        return point_tag_store.to_key_value_dict(self.index)

    def bounds(self):
        """
        Compute the bounding Rectangle of this Point.
        """
        return self.as_location().bounds()

    def intersects(self, polygon):
        """
        Check if this Point intersects some Polygon.
        """
        return self.as_location().intersects(polygon)

    def relations(self):
        """
        Get the frozenset of Relations of which this Point is a member.
        Returns an empty set if this Point is not a member of any Relations.
        """
        relation_map = self.get_parent_atlas()._get_pointIndexToRelationIndices()
        return self._get_relations_helper(relation_map, self.index)

    def get_type(self):
        """
        Overrides superclass get_type(). Always returns EntityType.POINT.
        """
        return EntityType.POINT


class Line(AtlasEntity):
    """
    An Atlas Line. Lines are non-navigable.
    """

    def __init__(self, parent_atlas, index):
        """
        Constuct a new Line. This should not be called directly.
        """
        super(Line, self).__init__(parent_atlas)
        self.index = index

    def __str__(self):
        """
        Get a string representation of this Line.
        """
        result = '[ '
        result += 'Line: id=' + str(self.get_identifier())
        result += ', geom=' + str(self.as_polyline())
        result += ', tags=' + str(self.get_tags())

        string = ''
        for relation in self.relations():
            string += str(relation.get_identifier()) + ','
        result += ', relations=[' + string + ']'

        result += ' ]'
        return result

    def as_polyline(self):
        """
        Get the PolyLine geometry of this Line.
        """
        return self.get_parent_atlas()._get_linePolyLines()[self.index]

    def get_identifier(self):
        """
        Get the Atlas identifier of this Line.
        """
        return self.get_parent_atlas()._get_lineIdentifiers().elements[self.index]

    def get_tags(self):
        """
        Get a dictionary of this Line's tags.
        """
        line_tag_store = self.get_parent_atlas()._get_lineTags()
        return line_tag_store.to_key_value_dict(self.index)

    def bounds(self):
        """
        Compute the bounding Rectangle of this Line.
        """
        return self.as_polyline().bounds()

    def intersects(self, polygon):
        """
        Check if this Line intersects some Polygon.
        """
        return self.as_polyline().intersects(polygon)

    def relations(self):
        """
        Get the frozenset of Relations of which this Line is a member.
        Returns an empty set if this Line is not a member of any Relations.
        """
        relation_map = self.get_parent_atlas()._get_lineIndexToRelationIndices()
        return self._get_relations_helper(relation_map, self.index)

    def get_type(self):
        """
        Overrides superclass get_type(). Always returns EntityType.LINE.
        """
        return EntityType.LINE


class Area(AtlasEntity):
    """
    An Atlas Area.
    """

    def __init__(self, parent_atlas, index):
        """
        Constuct a new Area. This should not be called directly.
        """
        super(Area, self).__init__(parent_atlas)
        self.index = index

    def __str__(self):
        """
        Get a string representation of this Area.
        """
        result = '[ '
        result += 'Area: id=' + str(self.get_identifier())
        result += ', geom=' + str(self.as_polygon())
        result += ', tags=' + str(self.get_tags())

        string = ''
        for relation in self.relations():
            string += str(relation.get_identifier()) + ','
        result += ', relations=[' + string + ']'

        result += ' ]'
        return result

    def get_identifier(self):
        """
        Get the Atlas identifier of this Area.
        """
        return self.get_parent_atlas()._get_areaIdentifiers().elements[self.index]

    def as_polygon(self):
        """
        Get the Polygon geometry of this Area.
        """
        return self.get_parent_atlas()._get_areaPolygons()[self.index]

    def get_tags(self):
        """
        Get a dictionary of this Area's tags.
        """
        area_tag_store = self.get_parent_atlas()._get_areaTags()
        return area_tag_store.to_key_value_dict(self.index)

    def bounds(self):
        """
        Compute the bounding Rectangle of this Area.
        """
        return self.as_polygon().bounds()

    def intersects(self, polygon):
        """
        Check if this Area intersects some Polygon.
        """
        return self.as_polygon().intersects(polygon)

    def relations(self):
        """
        Get the frozenset of Relations of which this Area is a member.
        Returns an empty set if this Area is not a member of any Relations.
        """
        relation_map = self.get_parent_atlas()._get_areaIndexToRelationIndices()
        return self._get_relations_helper(relation_map, self.index)

    def get_type(self):
        """
        Overrides superclass get_type(). Always returns EntityType.AREA.
        """
        return EntityType.AREA


class Node(AtlasEntity):
    """
    An Atlas Node. A Node is like a Point, except it is part of a navigable
    structure that can be traversed using the Node and Edge API methods.
    """

    def __init__(self, parent_atlas, index):
        """
        Constuct a new Node. This should not be called directly.
        """
        super(Node, self).__init__(parent_atlas)
        self.index = index

    def __str__(self):
        """
        Get a string representation of this Node.
        """
        result = '[ '
        result += 'Node: id=' + str(self.get_identifier())
        result += ', geom=' + str(self.as_location())
        result += ', tags=' + str(self.get_tags())

        string = ""
        for edge in self.in_edges():
            string += str(edge.get_identifier()) + ','
        result += ', inEdges=[' + string + ']'

        string = ""
        for edge in self.out_edges():
            string += str(edge.get_identifier()) + ','
        result += ', outEdges=[' + string + ']'

        string = ''
        for relation in self.relations():
            string += str(relation.get_identifier()) + ','
        result += ', relations=[' + string + ']'

        result += ' ]'
        return result

    def get_identifier(self):
        """
        Get the Atlas identifier of this node.
        """
        return self.get_parent_atlas()._get_nodeIdentifiers().elements[self.index]

    def as_location(self):
        """
        Get the Location of this Node.
        """
        long_location = self.get_parent_atlas()._get_nodeLocations().elements[self.index]
        return pyatlas.geometry.location_from_packed_int(long_location)

    def get_tags(self):
        """
        Get a dictionary of this Node's tags.
        """
        node_tag_store = self.get_parent_atlas()._get_nodeTags()
        return node_tag_store.to_key_value_dict(self.index)

    def bounds(self):
        """
        Compute the bounding Rectangle of this Point.
        """
        return self.as_location().bounds()

    def intersects(self, polygon):
        """
        Check if this Node intersects some Polygon.
        """
        return self.as_location().intersects(polygon)

    def in_edges(self):
        """
        Get a list of incoming Edges to this Node. The list is sorted by the
        Edges' Atlas IDs.
        """
        result = []
        node_in_edges_indices = self.get_parent_atlas()._get_nodeInEdgesIndices()
        for index in node_in_edges_indices.arrays[self.index].elements:
            result.append(Edge(self.get_parent_atlas(), index))
        return sorted(result)

    def out_edges(self):
        """
        Get a list of outgoing Edges from this Node. The list is sorted by the
        Edges' Atlas IDs.
        """
        result = []
        node_out_edges_indices = self.get_parent_atlas()._get_nodeOutEdgesIndices()
        for index in node_out_edges_indices.arrays[self.index].elements:
            result.append(Edge(self.get_parent_atlas(), index))
        return sorted(result)

    def connected_edges(self):
        """
        Get a list of all Edges connected to this Node. The list is sorted by
        the Edges' Atlas IDs.
        """
        result = []
        for edge in self.in_edges():
            result.append(edge)
        for edge in self.out_edges():
            result.append(edge)
        return sorted(result)

    def get_absolute_valence(self):
        """
        Get the number of Edges connected to this node. Considers all Edges, not
        just master Edges.
        """
        return len(self.connected_edges())

    def get_valence(self):
        """
        Get the number of Edges connected to this node. Only considers the
        master Edges.
        """
        connected_edges = self.connected_edges()
        valence = 0
        for edge in connected_edges:
            if edge.is_master_edge():
                valence += 1
        return valence

    def relations(self):
        """
        Get the frozenset of Relations of which this Node is a member.
        Returns an empty set if this Node is not a member of any Relations.
        """
        relation_map = self.get_parent_atlas()._get_nodeIndexToRelationIndices()
        return self._get_relations_helper(relation_map, self.index)

    def get_type(self):
        """
        Overrides superclass get_type(). Always returns EntityType.NODE.
        """
        return EntityType.NODE


class Edge(AtlasEntity):
    """
    A unidirectional Atlas Edge. An Edge is like a Line, except it is part of a
    navigable structure that can be traversed using the Node and Edge API methods.

    Bidirectional OSM ways are represented with two opposing Edges, where one
    of them is the master Edge. The master Edge will have a positive identifier
    and the same traffic direction as OSM.
    """

    def __init__(self, parent_atlas, index):
        """
        Constuct a new Edge. This should not be called directly.
        """
        super(Edge, self).__init__(parent_atlas)
        self.index = index

    def __str__(self):
        """
        Get a string representation of this Edge.
        """
        result = '[ '
        result += 'Edge: id=' + str(self.get_identifier())

        string = ""
        string += str(self.start().get_identifier()) + ','
        result += ', start=[' + string + ']'

        string = ""
        string += str(self.end().get_identifier()) + ','
        result += ', end=[' + string + ']'

        result += ', geom=' + str(self.as_polyline())
        result += ', tags=' + str(self.get_tags())

        string = ''
        for relation in self.relations():
            string += str(relation.get_identifier()) + ','
        result += ', relations=[' + string + ']'

        result += ' ]'
        return result

    def __lt__(self, other):
        """
        Custom implementation of less-than so that collections of Edges can be
        easily sorted.
        """
        return self.get_identifier() < other.get_identifier()

    def as_polyline(self):
        """
        Get the PolyLine geometry of this Edge.
        """
        return self.get_parent_atlas()._get_edgePolyLines()[self.index]

    def get_identifier(self):
        """
        Get the Atlas identifier of this Edge.
        :return: the Atlas id
        """
        return self.get_parent_atlas()._get_edgeIdentifiers().elements[self.index]

    def get_tags(self):
        """
        Get a dictionary of this Edge's tags.
        """
        edge_tag_store = self.get_parent_atlas()._get_edgeTags()
        return edge_tag_store.to_key_value_dict(self.index)

    def bounds(self):
        """
        Compute the bounding Rectangle of this Edge.
        """
        return self.as_polyline().bounds()

    def intersects(self, polygon):
        """
        Check if this Edge intersects some Polygon.
        """
        return self.as_polyline().intersects(polygon)

    def relations(self):
        """
        Get the frozenset of Relations of which this Edge is a member.
        Returns an empty set if this Edge is not a member of any Relations.
        """
        relation_map = self.get_parent_atlas()._get_edgeIndexToRelationIndices()
        return self._get_relations_helper(relation_map, self.index)

    def connected_nodes(self):
        """
        Get a frozenset of the Nodes connected to this Edge.
        """
        result = set()
        result.add(self.start())
        result.add(self.end())
        return frozenset(result)

    def connected_edges(self):
        """
        Get a frozenset of the Edges connected at the ends of the Nodes of this
        Edge. The set will not contain the Edge this method is called on, but
        will contain the reversed Edge if this Edge is part of a two-way road.
        """
        result = set()
        for edge in self.end().connected_edges():
            if self != edge:
                result.add(edge)
        for edge in self.start().connected_edges():
            if self != edge:
                result.add(edge)
        return result

    def start(self):
        """
        Get the starting Node of this Edge.
        """
        edge_start_node_index = self.get_parent_atlas()._get_edgeStartNodeIndex()
        index = edge_start_node_index.elements[self.index]
        return Node(self.get_parent_atlas(), index)

    def end(self):
        """
        Get the ending Node of this Edge.
        """
        edge_end_node_index = self.get_parent_atlas()._get_edgeEndNodeIndex()
        index = edge_end_node_index.elements[self.index]
        return Node(self.get_parent_atlas(), index)

    def get_master_edge(self):
        """
        Get the master for this Edge. Returns itself if this is the master Edge.
        """
        if self.is_master_edge():
            return self
        else:
            return self.get_parent_atlas().edge(-1 * self.get_identifier())

    def is_master_edge(self):
        """
        Checks if this Edge is a master edge.
        """
        return self.get_identifier() > 0

    def has_reversed_edge(self):
        """
        Checks if this Edge is a member of a bidirectional Edge pairing.
        """
        return self.get_parent_atlas().edge(-1 * self.get_identifier()) is not None

    def is_reversed_edge(self, candidate):
        """
        Check if the candidate Edge is the bidirectional reverse of this Edge.
        """
        if candidate.get_type() != EntityType.EDGE:
            return False
        return self.get_identifier() == -1 * candidate.get_identifier()

    def get_reversed_edge(self):
        """
        Get the bidirectional pair Edge to this Edge, if it exists. Returns None
        if it does not.
        """
        if not self.has_reversed_edge():
            return None
        return self.get_parent_atlas().edge(-1 * self.get_identifier())

    def get_highway_tag_value(self):
        """
        Get the value of the "highway" tag of this Edge, if present. Returns
        None if there is no "highway" tag.
        """
        tags = self.get_tags()
        if 'highway' in tags:
            return tags['highway']
        else:
            return None

    def is_connected_at_end_to(self, candidates):
        """
        Given a set of AtlasEntity candidates, test if this edge is directly
        connected at its end to at least one of the candidates.
        """
        for entity in candidates:
            if entity.get_type() == EntityType.NODE:
                if self.end() == entity:
                    return True
            if entity.get_type() == EntityType.EDGE:
                if self.end() == entity.start():
                    return True
        return False

    def is_connected_at_start_to(self, candidates):
        """
        Given a set of AtlasEntity candidates, test if this edge is directly
        connected at its start to at least one of the candidates.
        """
        for entity in candidates:
            if entity.get_type() == EntityType.NODE:
                if self.start() == entity:
                    return True
            if entity.get_type() == EntityType.EDGE:
                if self.start() == entity.end():
                    return True
        return False

    def is_way_sectioned(self):
        """
        Determine if this Edge is a way-sectioned road.
        """
        return pyatlas.identifier_converters.get_way_section_index(self.get_identifier()) != 0

    def get_type(self):
        """
        Overrides superclass get_type(). Always returns EntityType.EDGE.
        """
        return EntityType.EDGE


class Relation(AtlasEntity):
    """
    An Atlas Relation. Aggregates AtlasEntities in a logical relationship.
    Can contain other Relations as members.
    """

    def __init__(self, parent_atlas, index):
        """
        Constuct a new Relation. This should not be called directly.
        """
        super(Relation, self).__init__(parent_atlas)
        self.index = index

    def __str__(self):
        """
        Get a string representation of this Relation.
        """
        result = '[ '
        result += 'Relation: id=' + str(self.get_identifier())

        string = ''
        for member in self.get_members():
            string += str(member) + ','
        result += ', members=[' + string + ']'

        string = ''
        for relation in self.relations():
            string += str(relation.get_identifier()) + ','
        result += ', relations=[' + string + ']'

        result += ', tags=' + str(self.get_tags())
        result += ' ]'
        return result

    def get_identifier(self):
        """
        Get the Atlas identifier of this Relation.
        """
        return self.get_parent_atlas()._get_relationIdentifiers().elements[self.index]

    def get_members(self):
        """
        Get a sorted list of this Relation's members. The members are in
        RelationMember form.
        """
        result = []
        relation_identifiers = self.get_parent_atlas()._get_relationIdentifiers()
        relation_member_types = self.get_parent_atlas()._get_relationMemberTypes()
        relation_member_indices = self.get_parent_atlas()._get_relationMemberIndices()
        relation_member_roles = self.get_parent_atlas()._get_relationMemberRoles()
        dictionary = self.get_parent_atlas()._get_dictionary()

        array_index = 0
        # the relationMemberTypes field is a byte array, so the Python treats
        # it as a string. We need to convert it to a true byte array.
        for type_value in bytearray(relation_member_types.arrays[self.index].elements):
            member_index = relation_member_indices.arrays[self.index].elements[array_index]
            role = dictionary.word(relation_member_roles.arrays[self.index].elements[array_index])

            if type_value == EntityType.NODE:
                entity = Node(self.get_parent_atlas(), member_index)
            elif type_value == EntityType.EDGE:
                entity = Edge(self.get_parent_atlas(), member_index)
            elif type_value == EntityType.AREA:
                entity = Area(self.get_parent_atlas(), member_index)
            elif type_value == EntityType.LINE:
                entity = Line(self.get_parent_atlas(), member_index)
            elif type_value == EntityType.POINT:
                entity = Point(self.get_parent_atlas(), member_index)
            elif type_value == EntityType.RELATION:
                entity = Relation(self.get_parent_atlas(), member_index)
            else:
                raise ValueError('invalid EntityType value ' + str(type_value))
            result.append(RelationMember(role, entity, relation_identifiers.elements[self.index]))
            array_index += 1

        return sorted(result)

    def get_tags(self):
        """
        Get a dictionary of this Relation's tags.
        """
        relation_tag_store = self.get_parent_atlas()._get_relationTags()
        return relation_tag_store.to_key_value_dict(self.index)

    def bounds(self):
        """
        Compute the bounding Rectangle of this Relation's members.
        """
        # FIXME this fails if Relations have self-referencing members
        # this will never happen in a PackedAtlas so it should be OK for now
        # if pyatlas ever supports MultiAtlas then this will be a concern

        members = self.get_members()
        if len(members) == 0:
            return pyatlas.geometry.Rectangle(0, 0)

        entities_to_consider = []
        for member in self.get_members():
            entity = member.get_entity()
            if entity is None:
                raise ValueError('entity was None, how did this happen?')
            entities_to_consider.append(entity)

        return pyatlas.geometry.bounds_atlasentities(entities_to_consider)

    def intersects(self, polygon):
        """
        Check if any member of this Relation intersects some Polygon.
        """
        # FIXME this fails if Relations have self-referencing members
        # this will never happen in a PackedAtlas so it should be OK for now
        # if pyatlas ever supports MultiAtlas then this will be a concern

        for member in self.get_members():
            entity = member.get_entity()
            if entity.intersects(polygon):
                return True

        return False

    def relations(self):
        """
        Get the frozenset of Relations of which this Relation is a member.
        Returns an empty set if this Relation is not a member of any Relations.
        """
        relation_map = self.get_parent_atlas()._get_relationIndexToRelationIndices()
        return self._get_relations_helper(relation_map, self.index)

    def get_type(self):
        """
        Overrides superclass get_type(). Always returns EntityType.RELATION.
        """
        return EntityType.RELATION


class RelationMember(object):
    """
    A container type for Relation members. A RelationMember has a role as well
    as a reference to its AtlasEntity.
    """

    def __init__(self, role, entity, identifier):
        """
        Create a new RelationMember.
        """
        self.role = role
        self.entity = entity
        self.identifier = identifier

    def __lt__(self, other):
        """
        Define an ordering for RelationMembers. Compare EntityTypes, then
        identifiers, then roles.
        """
        if self.entity.get_type() < other.entity.get_type():
            return True
        elif self.entity.get_type() > other.entity.get_type():
            return False
        else:
            if self.identifier < other.identifier:
                return True
            elif self.identifier > other.identifier:
                return False
            else:
                if self.role < other.role:
                    return True
                else:
                    return False

    def __str__(self):
        """
        Get a string representation of this RelationMember.
        """
        result = '[ '
        result += 'id=' + str(self.get_entity().get_identifier())
        result += ', type=' + entity_type_to_str(self.entity.get_type())
        result += ', role=' + str(self.get_role())
        result += ' ]'
        return result

    def get_entity(self):
        """
        Get this RelationMember's AtlasEntity.
        """
        return self.entity

    def get_relation_identifier(self):
        """
        Get the identifier of the Relation of which this RelationMember is a member.
        """
        return self.identifier

    def get_role(self):
        """
        Get the role of this RelationMember.
        """
        return self.role


def entity_type_to_str(value):
    """
    Convert an EntityType enum to a string representation.
    """
    return EntityType._strs[value]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProtoPackedTagStore.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import ProtoIntegerArrayOfArrays_pb2 as ProtoIntegerArrayOfArrays__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ProtoPackedTagStore.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\032ProtoPackedTagStoreWrapperP\001',
  serialized_pb=b'\n\x19ProtoPackedTagStore.proto\x12\x1dorg.openstreetmap.atlas.proto\x1a\x1fProtoIntegerArrayOfArrays.proto\"\xb6\x01\n\x13ProtoPackedTagStore\x12\r\n\x05index\x18\x01 \x01(\x03\x12\x46\n\x04keys\x18\x02 \x01(\x0b\x32\x38.org.openstreetmap.atlas.proto.ProtoIntegerArrayOfArrays\x12H\n\x06values\x18\x03 \x01(\x0b\x32\x38.org.openstreetmap.atlas.proto.ProtoIntegerArrayOfArraysB\x1e\x42\x1aProtoPackedTagStoreWrapperP\x01'
  ,
  dependencies=[ProtoIntegerArrayOfArrays__pb2.DESCRIPTOR,])




_PROTOPACKEDTAGSTORE = _descriptor.Descriptor(
  name='ProtoPackedTagStore',
  full_name='org.openstreetmap.atlas.proto.ProtoPackedTagStore',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='index', full_name='org.openstreetmap.atlas.proto.ProtoPackedTagStore.index', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keys', full_name='org.openstreetmap.atlas.proto.ProtoPackedTagStore.keys', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values', full_name='org.openstreetmap.atlas.proto.ProtoPackedTagStore.values', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=94,
  serialized_end=276,
)

_PROTOPACKEDTAGSTORE.fields_by_name['keys'].message_type = ProtoIntegerArrayOfArrays__pb2._PROTOINTEGERARRAYOFARRAYS
_PROTOPACKEDTAGSTORE.fields_by_name['values'].message_type = ProtoIntegerArrayOfArrays__pb2._PROTOINTEGERARRAYOFARRAYS
DESCRIPTOR.message_types_by_name['ProtoPackedTagStore'] = _PROTOPACKEDTAGSTORE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoPackedTagStore = _reflection.GeneratedProtocolMessageType('ProtoPackedTagStore', (_message.Message,), {
  'DESCRIPTOR' : _PROTOPACKEDTAGSTORE,
  '__module__' : 'ProtoPackedTagStore_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoPackedTagStore)
  })
_sym_db.RegisterMessage(ProtoPackedTagStore)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProtoAtlas.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Point_pb2 as Point__pb2
from . import Line_pb2 as Line__pb2
from . import Area_pb2 as Area__pb2
from . import Node_pb2 as Node__pb2
from . import Edge_pb2 as Edge__pb2
from . import Relation_pb2 as Relation__pb2
from . import ProtoAtlasMetaData_pb2 as ProtoAtlasMetaData__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ProtoAtlas.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\021ProtoAtlasWrapperP\001',
  serialized_pb=b'\n\x10ProtoAtlas.proto\x12\x1dorg.openstreetmap.atlas.proto\x1a\x0bPoint.proto\x1a\nLine.proto\x1a\nArea.proto\x1a\nNode.proto\x1a\nEdge.proto\x1a\x0eRelation.proto\x1a\x18ProtoAtlasMetaData.proto\"\xc0\x04\n\nProtoAtlas\x12\x16\n\x0enumberOfPoints\x18\x01 \x01(\x03\x12\x39\n\x06points\x18\x02 \x03(\x0b\x32).org.openstreetmap.atlas.proto.ProtoPoint\x12\x15\n\rnumberOfLines\x18\x03 \x01(\x03\x12\x37\n\x05lines\x18\x04 \x03(\x0b\x32(.org.openstreetmap.atlas.proto.ProtoLine\x12\x15\n\rnumberOfAreas\x18\x05 \x01(\x03\x12\x37\n\x05\x61reas\x18\x06 \x03(\x0b\x32(.org.openstreetmap.atlas.proto.ProtoArea\x12\x15\n\rnumberOfNodes\x18\x07 \x01(\x03\x12\x37\n\x05nodes\x18\x08 \x03(\x0b\x32(.org.openstreetmap.atlas.proto.ProtoNode\x12\x15\n\rnumberOfEdges\x18\t \x01(\x03\x12\x37\n\x05\x65\x64ges\x18\n \x03(\x0b\x32(.org.openstreetmap.atlas.proto.ProtoEdge\x12\x19\n\x11numberOfRelations\x18\x0b \x01(\x03\x12?\n\trelations\x18\x0c \x03(\x0b\x32,.org.openstreetmap.atlas.proto.ProtoRelation\x12\x43\n\x08metaData\x18\r \x01(\x0b\x32\x31.org.openstreetmap.atlas.proto.ProtoAtlasMetaDataB\x15\x42\x11ProtoAtlasWrapperP\x01'
  ,
  dependencies=[Point__pb2.DESCRIPTOR,Line__pb2.DESCRIPTOR,Area__pb2.DESCRIPTOR,Node__pb2.DESCRIPTOR,Edge__pb2.DESCRIPTOR,Relation__pb2.DESCRIPTOR,ProtoAtlasMetaData__pb2.DESCRIPTOR,])




_PROTOATLAS = _descriptor.Descriptor(
  name='ProtoAtlas',
  full_name='org.openstreetmap.atlas.proto.ProtoAtlas',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='numberOfPoints', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.numberOfPoints', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='points', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.points', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numberOfLines', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.numberOfLines', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lines', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.lines', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numberOfAreas', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.numberOfAreas', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='areas', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.areas', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numberOfNodes', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.numberOfNodes', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nodes', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.nodes', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numberOfEdges', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.numberOfEdges', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='edges', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.edges', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numberOfRelations', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.numberOfRelations', index=10,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relations', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.relations', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metaData', full_name='org.openstreetmap.atlas.proto.ProtoAtlas.metaData', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=155,
  serialized_end=731,
)

_PROTOATLAS.fields_by_name['points'].message_type = Point__pb2._PROTOPOINT
_PROTOATLAS.fields_by_name['lines'].message_type = Line__pb2._PROTOLINE
_PROTOATLAS.fields_by_name['areas'].message_type = Area__pb2._PROTOAREA
_PROTOATLAS.fields_by_name['nodes'].message_type = Node__pb2._PROTONODE
_PROTOATLAS.fields_by_name['edges'].message_type = Edge__pb2._PROTOEDGE
_PROTOATLAS.fields_by_name['relations'].message_type = Relation__pb2._PROTORELATION
_PROTOATLAS.fields_by_name['metaData'].message_type = ProtoAtlasMetaData__pb2._PROTOATLASMETADATA
DESCRIPTOR.message_types_by_name['ProtoAtlas'] = _PROTOATLAS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoAtlas = _reflection.GeneratedProtocolMessageType('ProtoAtlas', (_message.Message,), {
  'DESCRIPTOR' : _PROTOATLAS,
  '__module__' : 'ProtoAtlas_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoAtlas)
  })
_sym_db.RegisterMessage(ProtoAtlas)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProtoAtlasMetaData.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Tag_pb2 as Tag__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ProtoAtlasMetaData.proto',
  package='org.openstreetmap.atlas.proto',
  syntax='proto2',
  serialized_options=b'B\031ProtoAtlasMetaDataWrapperP\001',
  serialized_pb=b'\n\x18ProtoAtlasMetaData.proto\x12\x1dorg.openstreetmap.atlas.proto\x1a\tTag.proto\"\xa8\x02\n\x12ProtoAtlasMetaData\x12\x12\n\nedgeNumber\x18\x01 \x01(\x03\x12\x12\n\nnodeNumber\x18\x02 \x01(\x03\x12\x12\n\nareaNumber\x18\x03 \x01(\x03\x12\x12\n\nlineNumber\x18\x04 \x01(\x03\x12\x13\n\x0bpointNumber\x18\x05 \x01(\x03\x12\x16\n\x0erelationNumber\x18\x06 \x01(\x03\x12\x10\n\x08original\x18\x07 \x01(\x08\x12\x13\n\x0b\x63odeVersion\x18\x08 \x01(\t\x12\x13\n\x0b\x64\x61taVersion\x18\t \x01(\t\x12\x0f\n\x07\x63ountry\x18\n \x01(\t\x12\x11\n\tshardName\x18\x0b \x01(\t\x12\x35\n\x04tags\x18\x0c \x03(\x0b\x32\'.org.openstreetmap.atlas.proto.ProtoTagB\x1d\x42\x19ProtoAtlasMetaDataWrapperP\x01'
  ,
  dependencies=[Tag__pb2.DESCRIPTOR,])




_PROTOATLASMETADATA = _descriptor.Descriptor(
  name='ProtoAtlasMetaData',
  full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='edgeNumber', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.edgeNumber', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nodeNumber', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.nodeNumber', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='areaNumber', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.areaNumber', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lineNumber', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.lineNumber', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pointNumber', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.pointNumber', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationNumber', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.relationNumber', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='original', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.original', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='codeVersion', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.codeVersion', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataVersion', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.dataVersion', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='country', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.country', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shardName', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.shardName', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='org.openstreetmap.atlas.proto.ProtoAtlasMetaData.tags', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=367,
)

_PROTOATLASMETADATA.fields_by_name['tags'].message_type = Tag__pb2._PROTOTAG
DESCRIPTOR.message_types_by_name['ProtoAtlasMetaData'] = _PROTOATLASMETADATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoAtlasMetaData = _reflection.GeneratedProtocolMessageType('ProtoAtlasMetaData', (_message.Message,), {
  'DESCRIPTOR' : _PROTOATLASMETADATA,
  '__module__' : 'ProtoAtlasMetaData_pb2'
  # @@protoc_insertion_point(class_scope:org.openstreetmap.atlas.proto.ProtoAtlasMetaData)
  })
_sym_db.RegisterMessage(ProtoAtlasMetaData)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
